//     The Brookshear Machine
//     Copyright (C) 2004 E. Rogan Creswick
//
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
* Commands.java
*
* Last edited on 3/11/2000 at 6:42 PM by: Rogan Creswick
*
* Revision history:
*
* 1/21/00 - modified code to allow for program counter to be used
*			properly.
* 1/29/00 - added exit() method to be called by file | exit menu item
* 1/31/00 - changed shiftAddressUp() and Insert() into one function.
*			added insert() and delete() and changed old ones to shiftInsert() and shiftDelete()
* 2/3/00  - added about() to display Globals.aboutWindow
* 2/6/00  - added getRegisters() and added call in step()
* 2/14/00 - added undo(), backupMemory(), shiftUndoDown() methods...and
*           best of all, they work!
* 2/15/00 - modified insert(), shiftinsert(), delete() and shiftdelete()
*           as marked in comments
* 3/11/00 - removed some methods which dealt with help system,
*			they can now be found in HelpSystem.java
**/


import java.util.StringTokenizer;
import java.awt.*;

public class Commands{
	public static void exit()
	{
		try{
			System.exit(0); // closes frame if application
		}catch(Exception ex){  // an exception is thrown if an Applet calls System.exit()
			Globals.rootObj.dispose(); // applet Frame is closed.
		}
	}// end exit()

	public static void open()
	{
		Globals.openWindow.ioPort();
	}// end open()

	public static void save()
	{
		//Globals.saveWindow.promptSave();
		Globals.mail = new Mailer();
	}// end save()

	public static void clear()
	{
		Globals.errorPrompt("This will erase all code!");
		if(!Globals.errorPromptBool)
		{
			return;
		}
		for(int i = 0; i < Globals.MEMSIZE * 2; i++)
		{
			Globals.MemoryAddress[i] = -1;
		}
		Globals.updateTextfields();
		//Globals.updateMemoryAddresses();
	}// end clear();
/*
	public static void toggleHelp()
	{
		Globals.mainGUI.showHelp();
	} // end showHelp()
*/
	public static void toggleMatrix()
	{
		//((CardLayout)(Globals.scrollPanePanel.getLayout())).next(Globals.scrollPanePanel);
		Globals.scrollPanePanel.next();

		getInput();
		Globals.matrixShowing = !Globals.matrixShowing;
		Globals.updateTextfields();
		//if (!Globals.matrixShowing)
		//	Globals.updateMemoryAddresses();

	} // end viewMatrix()

	public static void reset()
	{
		for (int i =0; i < Globals.REGSIZE * 2; i++)
		{
			Globals.RegisterAddress[i] = 0;
		}
		for (int i =0; i < Globals.REGSIZE; i++)
		{
			Globals.registerOutputLabels[i].setText("00");
		}
		Globals.ProgramCounter = 0;
		Globals.programCounterField.setText("00");
	} // end reset()

	public static void run()
	{
		/*if (!Globals.isRunning)
		{
			Globals.ProgramCounter = Globals.toInt(Globals.programCounterField.getText());
		}
		if (Globals.ProgramCounter < Globals.MEMSIZE)
		{
			do{
				step();
			}while(Globals.ProgramCounter < Globals.MEMSIZE && Globals.isRunning);
		}*/

		Globals.isThreaded = true;
		Globals.mainGUI.run.setEnabled(false);
		Globals.programRunner = new CompilerThread();
		Globals.programRunner.start();

	} // end run()

	public static void step()
	{
		if (!Globals.isRunning)
		{
			Globals.mainGUI.stop.setEnabled(true);

			Globals.programCounterField.setEditable(false);
			Globals.ProgramCounter = Globals.toInt(Globals.programCounterField.getText());
			for(int i = 0; i < Globals.REGSIZE; i++)
			{
				Globals.registerOutputLabels[i].setEditable(false);
			}
			for(int i = 0; i < Globals.MEMSIZE/2; i++)
			{
				Globals.memoryInputFields[i].setEditable(false);
			}
			for(int i = 0; i < Globals.MEMSIZE; i++)
			{
				Globals.matrixFields[i].setEditable(false);
			}
			getRegisters();
			getInput();
			// back up memory
			backupMemory();
			/*for(int i = 0; i < Globals.MEMSIZE*2; i++)
			{
				Globals.MemoryBackup[i] = Globals.MemoryAddress[i];
			}*/
			Globals.stopProgram = false;
			Globals.isRunning = true;
		}

		if (Globals.ProgramCounter < Globals.MEMSIZE)
		{
			Highlight.unHighlight();
			Highlight.highlight(Globals.ProgramCounter / 2);

			Operators.ProcessOperator(Globals.ProgramCounter / 2);
			if (Globals.stopProgram)
			{
				stop();
			}
		}
		else
			stop();
	} // end step()

	public static void stop()	// renamed to halt
	{
		if (Globals.programRunner != null)
		{
			Globals.isThreaded = false;
			Globals.programRunner = null;
			Globals.mainGUI.run.setEnabled(true);
		}

		Globals.isRunning = false;
		Globals.mainGUI.stop.setEnabled(false);
		Globals.programCounterField.setEditable(true);
		Highlight.unHighlight();

		// reset memory values to initial values
		/*for(int i = 0; i < Globals.MEMSIZE*2; i++)
		{
			Globals.MemoryAddress[i] = Globals.MemoryBackup[i];
		}*/
		undo(false);
		//Globals.updateTextfields();
		for(int i = 0; i < Globals.REGSIZE; i++)
		{
			Globals.registerOutputLabels[i].setEditable(true);
		}
		for(int i = 0; i < Globals.MEMSIZE/2; i++)
		{
			Globals.memoryInputFields[i].setEditable(true);
		}
		for(int i = 0; i < Globals.MEMSIZE; i++)
		{
			Globals.matrixFields[i].setEditable(true);
		}

	} // end stop()

	public static void getRegisters()
	{
		System.out.println("getRegisters() run");
		// reset registers
		for (int i =0; i < Globals.REGSIZE * 2; i++)
		{
			Globals.RegisterAddress[i] = -1;
		}

		int index, memIndex;
		String input;
		for (int arrayIndex = 0; arrayIndex < Globals.REGSIZE; arrayIndex++)
		{
			// input values for memory addresses
			input = Globals.registerOutputLabels[arrayIndex].getText();
			index = 0;
			memIndex = arrayIndex * 2;
			if (index < input.length() && input.charAt(index) != '\n')
			{
				Globals.RegisterAddress[memIndex] = Globals.toInt(input.charAt(index));
				System.out.println("Register Address " + Globals.toHex(memIndex) + " loaded with " + Globals.toHex(Globals.RegisterAddress[memIndex]));
			}
			else
			{
				continue;
			}
			index++;
			if (index < input.length() && input.charAt(index) != '\n')
			{
				Globals.RegisterAddress[memIndex+1] = Globals.toInt(input.charAt(index));
				System.out.println("Register Address " + Globals.toHex(memIndex + 1) + " loaded with " + Globals.toHex(Globals.RegisterAddress[memIndex+1]));
			}
		}
	} // end getRegisters()

	public static void getInput()
	{
		// reset Programcounter, registers, and memory addresses
		for (int i =0; i < Globals.MEMSIZE * 2; i++)
		{
			Globals.MemoryAddress[i] = -1;
		}

		// matrix showing, so get input from the matrix
		if(Globals.matrixShowing)
		{
			System.out.println("getInput() run and matrix is showing");
			int index, memIndex;
			String input;
			for (int arrayIndex = 0; arrayIndex < Globals.MEMSIZE; arrayIndex++)
			{
				// input values for memory addresses
				input = Globals.matrixFields[arrayIndex].getText();
				index = 0;
				memIndex = arrayIndex * 2;
				if (index < input.length() && input.charAt(index) != '\n')
				{
					Globals.MemoryAddress[memIndex] = Globals.toInt(input.charAt(index));
					//System.out.println("Memory Address " + Globals.toHex(memIndex) + " loaded with " + Globals.toHex(Globals.MemoryAddress[memIndex]));
				}
				else
				{
					continue;
				}
				index++;
				if (index < input.length() && input.charAt(index) != '\n')
				{
					Globals.MemoryAddress[memIndex+1] = Globals.toInt(input.charAt(index));
					//System.out.println("Memory Address " + Globals.toHex(memIndex + 1) + " loaded with " + Globals.toHex(Globals.MemoryAddress[memIndex+1]));
				}
			}
		}
		else	// matrix is not showing
		{
			int memIndex;
			String input;
			for (int arrayIndex = 0; arrayIndex < Globals.MEMSIZE / 2; arrayIndex++)
			{
				// input values for memory addresses
				input = Globals.memoryInputFields[arrayIndex].getText();
				memIndex = arrayIndex * 4;

				if (input.length() > 0 && input.charAt(0) != '\n')
				{
					Globals.MemoryAddress[memIndex] = Globals.toInt(input.charAt(0));
					//System.out.println("Memory Address " + Globals.toHex(memIndex) + " loaded with " + Globals.toHex(Globals.MemoryAddress[memIndex]));
				}
				else
					continue;
				if (input.length() > 1 && input.charAt(1) != '\n')
				{
					Globals.MemoryAddress[memIndex+1] = Globals.toInt(input.charAt(1));
					//System.out.println("Memory Address " + Globals.toHex(memIndex + 1) + " loaded with " + Globals.toHex(Globals.MemoryAddress[memIndex+1]));
				}
				else
					continue;
				if (input.length() > 2 && input.charAt(2) != '\n')
				{
					Globals.MemoryAddress[memIndex+2] = Globals.toInt(input.charAt(2));
					//System.out.println("Memory Address " + Globals.toHex(memIndex + 2) + " loaded with " + Globals.toHex(Globals.MemoryAddress[memIndex+2]));
				}
				else
					continue;
				if (input.length() > 3 && input.charAt(3) != '\n')
				{
					Globals.MemoryAddress[memIndex+3] = Globals.toInt(input.charAt(3));
					//System.out.println("Memory Address " + Globals.toHex(memIndex + 3) + " loaded with " + Globals.toHex(Globals.MemoryAddress[memIndex+3]));
				}
			}
		}
		//Globals.updateMemoryAddresses();
	} // end getInput()

	public static void getInput(int arrayIndex)
	{
		boolean illegalChar = false;
		byte newInt;
		if(Globals.matrixShowing)
		{
			// input values for memory addresses
			String input = Globals.memoryInputFields[arrayIndex].getText();

			if (input.length() > 0 && input.charAt(0) != '\n')
			{
				newInt = Globals.toInt(input.charAt(0));
				if (newInt == -1)
					illegalChar = true;
				Globals.MemoryAddress[arrayIndex*2] = newInt;
				//System.out.println("Memory Address " + Globals.toHex(arrayIndex) + " loaded with " + Globals.toHex(newInt));
			}
			else
			{
				Globals.MemoryAddress[arrayIndex*2] = -1;
			}
			if (input.length() > 1 && input.charAt(1) != '\n')
			{
				newInt = Globals.toInt(input.charAt(1));
				if (newInt == -1)
					illegalChar = true;
				Globals.MemoryAddress[arrayIndex*2+1] = newInt;
				//System.out.println("Memory Address " + Globals.toHex(arrayIndex + 1) + " loaded with " + Globals.toHex(newInt));
			}
			else
			{
				Globals.MemoryAddress[arrayIndex*2+1] = -1;
			}
		}

		if(!Globals.matrixShowing)
		{
			// input values for memory addresses
			String input = Globals.memoryInputFields[arrayIndex].getText();

			if (input.length() > 0 && input.charAt(0) != '\n')
			{
				newInt = Globals.toInt(input.charAt(0));
				if (newInt == -1)
					illegalChar = true;
				Globals.MemoryAddress[arrayIndex*4] = newInt;
				//System.out.println("Memory Address " + Globals.toHex(arrayIndex*2) + " loaded with " + Globals.toHex(newInt));
			}
			else
			{
				Globals.MemoryAddress[arrayIndex*4] = -1;
			}
			if (input.length() > 1 && input.charAt(1) != '\n')
			{
				newInt = Globals.toInt(input.charAt(1));
				if (newInt == -1)
					illegalChar = true;
				Globals.MemoryAddress[arrayIndex*4+1] = newInt;
				//System.out.println("Memory Address " + Globals.toHex(arrayIndex*2 + 1) + " loaded with " + Globals.toHex(newInt));
			}
			else
			{
				Globals.MemoryAddress[arrayIndex*4 + 1] = -1;
			}
			if (input.length() > 2 && input.charAt(2) != '\n')
			{
				newInt = Globals.toInt(input.charAt(2));
				if (newInt == -1)
					illegalChar = true;
				Globals.MemoryAddress[arrayIndex*4+2] = newInt;
				//System.out.println("Memory Address " + Globals.toHex(arrayIndex*2 + 2) + " loaded with " + Globals.toHex(newInt));
			}
			else
			{
				Globals.MemoryAddress[arrayIndex*4 + 2] = -1;
			}
			if (input.length() > 3 && input.charAt(3) != '\n')
			{
				newInt = Globals.toInt(input.charAt(3));
				if (newInt == -1)
					illegalChar = true;
				Globals.MemoryAddress[arrayIndex*4+3] = newInt;

				//System.out.println("Memory Address " + Globals.toHex(arrayIndex*2 + 3) + " loaded with " + Globals.toHex(newInt));
			}
			else
			{
				Globals.MemoryAddress[arrayIndex*4 + 3] = -1;
			}
			Output.displayOutputText(arrayIndex);
		}
		if (illegalChar)
		{
			Globals.bottomOutputField.setForeground(OurColors.bottomOutputFieldForeHighlight);
			Globals.bottomOutputField.setText("Illegal Character detected");
		}
		else
		{
			Globals.bottomOutputField.setText("");
			Globals.bottomOutputField.setForeground(OurColors.bottomOutputFieldFore);
		}
		//Globals.updateMemoryAddresses(arrayIndex);
	} // end getInput(int arrayIndex)

	public static void getInput(String inputString)
	{
		// input values for memory addresses
		for (int i =0; i < Globals.MEMSIZE * 2; i++)
		{
			Globals.MemoryAddress[i] = -1;
		}

		int index = -1;
		int memIndex = -1;
		while(true)
		{
			// if not first time through, will skip to next newline character
			while(index != -1 && index < inputString.length() && inputString.charAt(index) != '\n')
			{
				index++;
			}
			if (index == -1)
				index++;
			// skips current character if newline character
			if (index < inputString.length() && inputString.charAt(index) == '\n')
			{
				index++;
			}
			// exit the while loop if index is greater than available characters
			if (index >= inputString.length())
				break;
			memIndex++;
			if (index < inputString.length() && inputString.charAt(index) != '\n')
			{
				Globals.MemoryAddress[4*memIndex] = Globals.toInt(inputString.charAt(index));
				//System.out.println("Memory Address " + Globals.toHex(4 * memIndex) + " loaded with " + Globals.toHex(Globals.MemoryAddress[4*memIndex]));
			}
			else
				continue;
			index++;
			if (index < inputString.length() && inputString.charAt(index) != '\n')
			{
				Globals.MemoryAddress[4*memIndex+1] = Globals.toInt(inputString.charAt(index));
				//System.out.println("Memory Address " + Globals.toHex(4 * memIndex + 1) + " loaded with " + Globals.toHex(Globals.MemoryAddress[4*memIndex+1]));
			}
			else
				continue;
			index++;
			if (index < inputString.length() && inputString.charAt(index) != '\n')
			{
				Globals.MemoryAddress[4*memIndex+2] = Globals.toInt(inputString.charAt(index));
				//System.out.println("Memory Address " + Globals.toHex(4 * memIndex + 2) + " loaded with " + Globals.toHex(Globals.MemoryAddress[4*memIndex+2]));
			}
			else
				continue;
			index++;
			if (index < inputString.length() && inputString.charAt(index) != '\n')
			{
				Globals.MemoryAddress[4*memIndex+3] = Globals.toInt(inputString.charAt(index));
				//System.out.println("Memory Address " + Globals.toHex(4 * memIndex + 3) + " loaded with " + Globals.toHex(Globals.MemoryAddress[4*memIndex+3]));
			}
			else
				continue;
			index++;
		}
	} // end getInput(String inputString)

	public static void insert()
	{
		System.out.println("insert() called");
		int memLocation;
		if(Globals.matrixShowing)
			memLocation = Globals.hasFocus >> 1;
		else
			memLocation = Globals.hasFocus;

		backupMemory();		/* Added by Stacy on 2/14/00 */

		Commands.getInput();
		// check for data loss in last memory address.
		if((Globals.MemoryAddress[Globals.MEMSIZE-2 * 2] != -1) || (Globals.MemoryAddress[(Globals.MEMSIZE-2 * 2) + 1] != -1)
			|| (Globals.MemoryAddress[(Globals.MEMSIZE-2 * 2) + 2] != -1) || (Globals.MemoryAddress[(Globals.MEMSIZE-2 * 2) + 3] != -1))
		{
			Globals.errorPrompt("WARNING: The last memory address contains data that will be lost.");
			if(!Globals.errorPromptBool)
			{
				return;
			}
		}

		for(int count = Globals.MEMSIZE * 2 - 8; count >= memLocation * 4; count-=4)
		{
			// copy the four hex characters to the next line
			for(int i = 0; i < 4; i++)
			{
				Globals.MemoryAddress[count+i+4] = Globals.MemoryAddress[count+i];
			}
		}
		for(int i = 0; i < 4; i++)
		{
			Globals.MemoryAddress[4*memLocation+i] = -1;
		}
		Globals.updateTextfields();
	}// end insert()

	public static void shiftInsert()
	{
		System.out.println("shiftInsert() called");
		int memLocation;
		if(Globals.matrixShowing)
			memLocation = Globals.hasFocus >> 1;
		else
			memLocation = Globals.hasFocus;

		backupMemory();		/* Added by Stacy on 2/14/00 */

		Commands.getInput();
		// check for data loss in last memory address.
		if((Globals.MemoryAddress[Globals.MEMSIZE-2 * 2] != 0) || (Globals.MemoryAddress[(Globals.MEMSIZE-2 * 2) + 1] != 0)
			|| (Globals.MemoryAddress[(Globals.MEMSIZE-2 * 2) + 2] != 0) || (Globals.MemoryAddress[(Globals.MEMSIZE-2 * 2) + 3] != 0))
		{
			Globals.errorPrompt("WARNING: The last memory address contains data that will be lost.");
			if(!Globals.errorPromptBool)
			{
				return;
			}
		}

		int instruction1, instruction3, instruction4;

		for(int count = Globals.MEMSIZE * 2 - 8; count >= 0; count-=4)
		{
			instruction1 = Globals.MemoryAddress[count];
			instruction3 = Globals.MemoryAddress[count + 2];
			instruction4 = Globals.MemoryAddress[count + 3];
			int destination = instruction3*16 + instruction4;

			if((instruction1 == 1) || (instruction1 == 3) || (instruction1 == 11))
			{
				if((destination >= memLocation) && (destination <= Globals.MEMSIZE-4))
				{
					destination +=4;
					Globals.MemoryAddress[count +2] = (byte)(destination >> 4);
					Globals.MemoryAddress[count +3] = (byte)(destination % 16);
				}
			}
		}
		for(int count = Globals.MEMSIZE * 2 - 8; count >= memLocation * 4; count-=4)
		{
			// copy the four hex characters to the next line
			for(int i = 0; i < 4; i++)
			{
				Globals.MemoryAddress[count+i+4] = Globals.MemoryAddress[count+i];
			}
		}
		for(int i = 0; i < 4; i++)
		{
			Globals.MemoryAddress[4*memLocation+i] = -1;
		}
		Globals.updateTextfields();
	}// end insert()

	public static void delete()
	{
		System.out.println("delete() called");
		int location;
		if(Globals.matrixShowing)
			location = Globals.hasFocus >> 1;
		else
			location = Globals.hasFocus;

		backupMemory();		/* Added by Stacy on 2/14/00 */

		Commands.getInput();

		int memLocation = location;

		for(int count = memLocation * 4 + 4; count < Globals.MEMSIZE * 2; count+=4)
		{
			// copy the four hex characters to the previous line
			for(int i = 0; i < 4; i++)
			{
				Globals.MemoryAddress[count+i-4] = Globals.MemoryAddress[count+i];
			}
		}
		// delete first 4 hex characters
		for(int i = 1; i < 5; i++)
		{
			Globals.MemoryAddress[Globals.MEMSIZE * 2 - i] = -1;
		}
		Globals.updateTextfields();
		//Globals.updateMemoryAddresses();
	}// end delete()

	public static void shiftDelete()
	{
		System.out.println("shiftDelete() called");
		int location;
		if(Globals.matrixShowing)
			location = Globals.hasFocus >> 1;
		else
			location = Globals.hasFocus;

		backupMemory();		/* Added by Stacy on 2/14/00 */

		Commands.getInput();

		int memLocation = location;
		int instruction1, instruction3, instruction4;

		for(int count = 0; count < Globals.MEMSIZE * 2; count+=4)
		{
			instruction1 = Globals.MemoryAddress[count];
			instruction3 = Globals.MemoryAddress[count + 2];
			instruction4 = Globals.MemoryAddress[count + 3];
			int destination = (instruction3 << 4) + instruction4;

			// shift the addresses 4 hex characters down
			if((instruction1 == 1) || (instruction1 == 3) || (instruction1 == 11))
			{
				if((destination >= memLocation) && (destination >= 4 ))
				{
					destination -=4;
					Globals.MemoryAddress[count + 2] = (byte)(destination >> 4);
					Globals.MemoryAddress[count + 3] = (byte)(destination % 16);
				}
			}
		}
		for(int count = memLocation * 4 + 4; count < Globals.MEMSIZE * 2; count+=4)
		{
			// copy the four hex characters to the previous line
			for(int i = 0; i < 4; i++)
			{
				Globals.MemoryAddress[count+i-4] = Globals.MemoryAddress[count+i];
			}
		}
		// delete first 4 hex characters
		for(int i = 1; i < 5; i++)
		{
			Globals.MemoryAddress[Globals.MEMSIZE * 2 - i] = -1;
		}
		Globals.updateTextfields();
	}// end shiftDelete()

	public static void undo(boolean update)
	{
		System.out.println("Current Undo Position at undo(): " + Globals.CurrentUndoPosition);
		if (Globals.CurrentUndoPosition != -1) {
			for (int i = 0; i < Globals.MEMSIZE*2; i++) {
				Globals.MemoryAddress[i] = Globals.MemoryBackup[Globals.CurrentUndoPosition][i];
			}
			if (Globals.CurrentUndoPosition > 0) {
				Globals.CurrentUndoPosition = Globals.CurrentUndoPosition - 1;
			}
			System.out.println("Current Undo Position after undo(): " + Globals.CurrentUndoPosition);
			if (update) {
				Globals.updateTextfields();
			}
		}
	} // end undo()

	public static void backupMemory()
	{
		System.out.println("Current Undo Position at backupMemory(): " + Globals.CurrentUndoPosition);
		if (Globals.CurrentUndoPosition == (Globals.UNDOLEVELS - 1)) {
			System.out.println("Shifting undos down a level.");
			shiftUndosDown();
		} else {
			System.out.println("Adding 1 to CurrentUndoPosition.");
			Globals.CurrentUndoPosition = Globals.CurrentUndoPosition + 1;
		}
		System.out.println("Saving Backup to Position: " + Globals.CurrentUndoPosition);
		for (int i = 0; i < Globals.MEMSIZE*2; i++)
		{
			Globals.MemoryBackup[Globals.CurrentUndoPosition][i] = Globals.MemoryAddress[i];
		}
	} // end backupMemory()

	public static void shiftUndosDown()
	{
		for (int i = 0; i < (Globals.UNDOLEVELS - 1); i++) {
			for (int j = 0; j < Globals.MEMSIZE*2; j++) {
				Globals.MemoryBackup[i][j] = Globals.MemoryBackup[i+1][j];
			}
		}
		Globals.CurrentUndoPosition = Globals.UNDOLEVELS - 1;
	} // end shiftUndosDown()

}// end class Commands
