//     The Brookshear Machine
//     Copyright (C) 2004 E. Rogan Creswick
//
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
* CompilerGUI.java
*
* Last edited on 4/28/2000 at 12:36 AM by Rogan Creswick
*
* Revision history:
*
* 1/21/00 - added color control through OurColors.java
*		  -	modified File menu, removed Clear button.
* 1/22/00 - added more color info
*		  - optimized initMatrix()
*		  - set initial textField to be first memory address.
*		  - changed stop to halt
* 1/23/00 - added autoscrolling.
* 2/2/00  - southTextField added
* 2/3/00  - Help | About added to menu system and linked to Commands.about()
* 2/6/00  - Removed Buttons
* 2/10/00 - Changed registerPanel to a gridbag and added the program counter here.
* 2/14/00 - Added tabPanels for matrix/list modes and help menu
*		  - Removed View menu
*		  - Added Edit | Undo menu option, linked to Commands.undo()
* 3/11/00 - Added slider
*		  - Removed Help system to be placed in a new class ( HelpSystem.java )
* 3/19/00 - Modified Help menu: added a close help option and fixed some errors with the other selections.
* 4/29/00 - reenabled Edit|Prefrences to open a PrefrencesDialog.
**/


import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class CompilerGUI extends Panel{
    ActionGrabber actionHandler = new ActionGrabber();
    AdjustmentGrabber adjustmentHandler = new AdjustmentGrabber();
    //ItemGrabber itemHandler     = new ItemGrabber();


    Frame rootFrame;

    private PreferencesDialog prefDialog;

    private ScrollPane memoryListScrollPane   = new ScrollPane();
    private ScrollPane memoryMatrixScrollPane = new ScrollPane();

    HelpSystem helpSys = new HelpSystem();


    Panel westPanel         = new Panel(); // Panel that holds CPU Registers and Program counter
    Panel centerPanel       = new Panel(); // Panel that holds Globals.scrollPanePanel and memory label.
    Panel registerPanel     = new Panel(); // Panel holding Registers in a GridLayout
    Panel memoryListPanel   = new Panel(); // Panel holding the memory as a list
    Panel memoryMatrixPanel = new Panel(); // Panel holding memory as a matrix
    Panel buttonPanel       = new Panel(); // Panel in the south area of main BorderLayout, has buttons
    Panel counterPanel	    = new Panel(); // Panel south of registerPanel, contains a label and counter
    Panel outputPanel	    = new Panel();
    //Panel helpPanel			= new Panel(); // help system Panel
    //Panel miniHelpPanel		= new Panel();
    //Panel miniHelpTopPanel	= new Panel();
    Panel speedSelectPanel= new Panel(); // panel to hold speed selection mechanisim.

    ScrollPane registerScrollPane = new ScrollPane();


    private MenuBar menuBar = new MenuBar();
    private Menu fileMenu	= new Menu("File");
    //private Menu viewMenu	= new Menu("View");
    private Menu helpMenu	= new Menu("Help");
    private Menu editMenu 	= new Menu("Edit");

    //private MenuItem fileNew    = new MenuItem("New");
    private MenuItem fileOpen   = new MenuItem("Import/Export");
    private MenuItem fileSave   = new MenuItem("Email Code");
    private MenuItem fileClear 	= new MenuItem("Clear");
    private MenuItem fileExit	= new MenuItem("Exit");

    private MenuItem editUndo			= new MenuItem("Undo Insert/Delete");
    private MenuItem editInsert			= new MenuItem("Insert");
    private MenuItem editDelete			= new MenuItem("Delete");
    private MenuItem editPreferences		= new MenuItem("Preferences");

    private MenuItem helpCommands		= new MenuItem("Commands");
    private MenuItem helpInfo			= new MenuItem("Compiler Info");
    private MenuItem helpAbout			= new MenuItem("About");
    private MenuItem helpClose			= new MenuItem("Close Help");

    Button run		= new Button("Run");
    Button stop		= new Button("Halt");
    Button step		= new Button("Step");
    Button reset	= new Button("Reset Registers/PC");

    Label slowSpeedLabel = new Label("Slow");
    Label fastSpeedLabel = new Label("Fast");
    Slider speedSlider = new Slider(Slider.HORIZONTAL, 125, 20);

    private Label registerPanelLabel		= new Label("CPU Registers:");
    private Label programCounterTitleLabela	= new Label("Program");
    private Label programCounterTitleLabelb	= new Label("Counter:");
    private Label memoryPanelLabel			= new Label("Main Memory / edit window");

    public CompilerGUI(Frame rootFrame){
	this.rootFrame = rootFrame;
	Globals.saveWindow = new SaveDialog(rootFrame);
	Globals.openWindow = new IODialog(rootFrame);
	prefDialog  = new PreferencesDialog(rootFrame);

	Globals.saveWindow.setBounds(rootFrame.getBounds().x+rootFrame.getBounds().width / 3,
				     rootFrame.getBounds().x+rootFrame.getBounds().height / 4,
				     100,100);
	//rootFrame.getBounds().width/3, 
	//rootFrame.getBounds().height/5);  //sets window size to proper size.
	Globals.openWindow.setBounds(rootFrame.getBounds().x+rootFrame.getBounds().width / 3,
				     rootFrame.getBounds().x+rootFrame.getBounds().height / 4,
				     230, 330);

	// rootFrame.getBounds().width/3, 
	//rootFrame.getBounds().height/3+100);
    }// end CompilerGUI()

    public void initWidgits(){
    
	initWest();
	initSpeedSelectPanel();
	initSouth();
	initMemoryList();
	initMatrix();
	initCenter();


	//***********************
	//	Menu setup
	//
	//fileMenu.add(fileNew);
	fileMenu.add(fileOpen);
	fileMenu.add(fileSave);
	fileMenu.addSeparator();
	fileMenu.add(fileClear);
	fileMenu.add(fileExit);

	editMenu.add(editUndo);
	editMenu.addSeparator();
	editMenu.add(editInsert);
	editMenu.add(editDelete);
	editMenu.addSeparator();
	editMenu.add(editPreferences);

	//viewMenu.add(viewType);
	helpClose.setEnabled(false);

	helpMenu.add(helpCommands);
	helpMenu.add(helpInfo);
	helpMenu.add(helpAbout);
	helpMenu.addSeparator();
	helpMenu.add(helpClose);

	menuBar.add(fileMenu);
	menuBar.add(editMenu);
	//menuBar.add(viewMenu);
	menuBar.add(helpMenu);

	rootFrame.setMenuBar(menuBar);
	//  End menu setup
	//**********************

	this.setLayout(new BorderLayout());


	this.add(westPanel, "West");
	this.add(centerPanel, "Center");
	this.add(buttonPanel, "South");

	setColors();

    }// end initWidgits()
    /*
      private void initHelp()
      {
      //TabPanel helpTabPanel = new TabPanel();

      GridBagLayout miniBag = new GridBagLayout();
      GridBagConstraints miniConstraints = new GridBagConstraints();

      Panel commandPanel 	= new Panel();
      Panel aboutPanel	= new Panel();
      Panel compilerPanel	= new Panel();

      aboutPanel.setLayout(new BorderLayout());
      commandPanel.setLayout(new BorderLayout());

      //**************************
      //  fill panels here

      TextArea mainArea = new TextArea("     A Typical Machine Language Compiler\n\n\tWritten by:\n       Stacy Brock\n       Rogan Creswick\n       Gary Hertel\n\n     A Typical Machine Language created by J. Glenn Brookshear, Marquette University.  The language can be found in _Computer_Scienc:_An_Overview_, by J. Glenn Brookshear.",0,0,TextArea.SCROLLBARS_VERTICAL_ONLY);
      TextArea commandArea = new TextArea("");
      commandArea.setText("Operand Codes\n\n"
      + "1: RXY: Load the register R with the bit pattern found in the memory cell whose address is XY\n"
      + "2: RXY: Load the register R with the bit pattern XY\n"
      + "3: RXY: Store the bit pattern found in register R in the memory cell whose address is XY\n"
      + "4: 0RS: Move the bit pattern found in register R to register S\n"
      + "5: RST: Add the bit pattern found in register S and T as though they were two's complement representations and leave the result in register R\n"
      + "6: RST: Add the bit patterns in registers S and T as though they represented values in the floatin-point notation and leave the floating point result in register R\n"
      + "7: RST: Or the bit patterns in registers S and T and place the result in register R\n8: RST: And the bit patterns in register S and T and place the result in register R\n"
      + "9: RST: Exclusive Or the bit patterns in registers S and T and place the result in register R\n"
      + "A: R0X: Rotate the bit pattern in register R one bit to the right X times.  Each time place the bit that started at the low-order end at the high order end\n"
      + "B: RXY: Jump to the instruction located in the memory cell at address XY if the bit pattern in register R is equal to the bit pattern in register number 0. Otherwise, continue with the normal sequence of execution.\n"
      + "C: Halt execution");

      commandPanel.add(commandArea, "Center");
      aboutPanel.add(mainArea, "Center");
      //
      //**************************

      helpTabPanel.add("Commands",commandPanel);
      helpTabPanel.add("Compiler",compilerPanel);
      helpTabPanel.add("About",aboutPanel);

      miniHelpTopPanel.setLayout(miniBag);
      miniHelpPanel.setLayout(new BorderLayout());

      helpPanel.setLayout(new BorderLayout());
      helpPanel.setVisible(false);

      miniHelpPanel.add(helpPanel,"Center");
      }// end initHelp();

    */
    private void initSpeedSelectPanel(){
	System.out.println(""+speedSlider.getPreferredSize().width);
	//speedSelectPanel.setSize(new Dimension(slowSpeedLabel.getPreferredSize().width+speedSlider.getPreferredSize().width + fastSpeedLabel.getPreferredSize().width,
	//	slowSpeedLabel.getPreferredSize().height));
	speedSelectPanel.setLayout(new BorderLayout());
	speedSelectPanel.add(slowSpeedLabel,"West");
	speedSelectPanel.add(speedSlider,"Center");
	speedSelectPanel.add(fastSpeedLabel,"East");

    }// end initSpeedSelectPanel()

    private void initSouth(){
	stop.setEnabled(false);

	buttonPanel.add(speedSelectPanel);

	buttonPanel.add(run);
	buttonPanel.add(step);
	buttonPanel.add(stop);
    }// end initSouth()

    private void initWest(){


	westPanel.setLayout(new BorderLayout());

	GridBagLayout registerBag = new GridBagLayout();
	GridBagConstraints regConstraints = new GridBagConstraints();
	registerPanel.setLayout(registerBag);

	for(int x=0; x < Globals.REGSIZE; x++){ // initalize the register Labels
	    
	    Globals.registerLabels[x] = new Label(Globals.toHex(x) + " ");
	    Globals.registerLabels[x].setFont(new Font("Courier",0,12));
	    Globals.registerLabels[x].setAlignment(Label.RIGHT);

	    Globals.registerOutputLabels[x] = new TextField("00");
	    Globals.registerOutputLabels[x].setFont(new Font("Courier",0,12));

	    regConstraints.gridx = 0;
	    regConstraints.gridy = x;
	    registerPanel.add(Globals.registerLabels[x],regConstraints);
	    regConstraints.gridx = 1;
	    registerPanel.add(Globals.registerOutputLabels[x],regConstraints);
	}

	regConstraints.gridx = 0;
	regConstraints.gridy = Globals.REGSIZE+1;

	registerPanel.add(programCounterTitleLabela,regConstraints);

	regConstraints.gridy = Globals.REGSIZE+2;

	registerPanel.add(programCounterTitleLabelb,regConstraints);

	regConstraints.gridx = 1;

	registerPanel.add(Globals.programCounterField,regConstraints);

	Panel tmpReg = new Panel(new BorderLayout()); // makes the Registers stay one size.
	tmpReg.add(registerPanel, "North");
	
	registerScrollPane.getVAdjustable().setUnitIncrement(25);
	registerScrollPane.add(tmpReg);

	westPanel.add(reset,"South");
	westPanel.add(registerPanelLabel,"North");
	//	westPanel.add(tmpReg,"West");
	westPanel.add(registerScrollPane,"West");
    }

    private void initCenter(){
	centerPanel.setLayout(new BorderLayout());

	Globals.scrollPanePanel.add("Memory List", memoryListScrollPane);
	Globals.scrollPanePanel.add("Memory Matrix", memoryMatrixScrollPane);

	centerPanel.add(Globals.scrollPanePanel,"Center");
	//centerPanel.add(memoryPanelLabel,"North");
	centerPanel.add(helpSys, "East");

	Globals.bottomOutputField.setEditable(false);
	centerPanel.add(Globals.bottomOutputField,"South");
    }// end initCenter()

    private void initMemoryList(){
	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints constraints = new GridBagConstraints();

	memoryListPanel.setLayout(gridBag);

	String tmp;

	constraints.gridx = 0;
	constraints.weightx = 0;
	constraints.gridheight = 1;
	constraints.gridwidth  = 1;
	constraints.insets = new Insets(0,8,0,0);
	// add the memory address labels
	for(int x=0; x < Globals.MEMSIZE/2 ; x++){ // fill in the memory address fields.
	    constraints.gridy = x;

	    tmp = Globals.toHex(x*2);
	    if(tmp.length() < 2)
		{
		    tmp = "0" + tmp;
		}
	    Globals.memoryAddressLabels[x] = new Label(tmp);
	    memoryListPanel.add(Globals.memoryAddressLabels[x], constraints);
	}

	constraints.gridx = 1;
	for(int x = 0; x < Globals.MEMSIZE/2; x++){
	    constraints.gridy = x;
	    Globals.memoryInputFields[x] = new TextField("",3);
	    Globals.memoryInputFields[x].setFont( new Font("Courier", 0, 12));
	    memoryListPanel.add(Globals.memoryInputFields[x], constraints);
	}

	constraints.anchor = GridBagConstraints.WEST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	constraints.gridx = 2;
	constraints.weightx = 100;

	for(int x = 0; x < Globals.MEMSIZE/2; x++){
	    
	    constraints.gridy = x;

	    Globals.outputLabels[x] = new FieldLabel("");
	    memoryListPanel.add(Globals.outputLabels[x], constraints);
	}

	memoryListScrollPane.getVAdjustable().setUnitIncrement(25);
	memoryListScrollPane.add(memoryListPanel);

    }// end initMemoryList()


    private void initMatrix(){
	memoryMatrixScrollPane.getVAdjustable().setUnitIncrement(25);
	memoryMatrixScrollPane.add(memoryMatrixPanel);

	int size = (int)Math.sqrt(Globals.MEMSIZE);	// number of rows/columns

	//**********************************
	//  GridBag Layout for Matrix Panel
	//
	GridBagLayout gridBag = new GridBagLayout();
	memoryMatrixPanel.setLayout(gridBag);

	GridBagConstraints constraints = new GridBagConstraints();
	constraints.gridy = 0;
	constraints.gridheight = 1;
	constraints.gridwidth  = 1;
	constraints.weighty = 0;

	for(int x = 0; x < size ; x++){ // initalize the top labels
	    Globals.topMatrixLabels[x] = new Label(Globals.toHex(x));
	    if(x == size - 1)
		{
		    constraints.weightx = 100;
		    constraints.anchor = constraints.WEST;
		}
	    else
		constraints.weightx = 0;

	    constraints.gridx = x + 1;
	    memoryMatrixPanel.add(Globals.topMatrixLabels[x], constraints);
	    Globals.topMatrixLabels[x].setAlignment(Label.CENTER);
	}

	constraints = new GridBagConstraints();
	constraints.gridheight = 1;
	constraints.gridwidth  = 1;
	constraints.gridx = 0;
	constraints.weightx = 0;
	constraints.insets = new Insets(0,8,0,0);

	for(int x = 0; x < size; x++){ // initalize the side labels
	    Globals.sideMatrixLabels[x] = new Label(Globals.toHex(x));
	    if(x == size - 1){
		constraints.weighty = 100;
		constraints.anchor = constraints.NORTH;
	    }else{
		constraints.weighty = 0;
	    }
	    constraints.gridy = x + 1;

	    memoryMatrixPanel.add(Globals.sideMatrixLabels[x], constraints);
	}

	for(int x = 0; x < Globals.MEMSIZE; x++){ // initalize the textFields
	    Globals.matrixFields[x] = new TextField(2);
	}
	int count = 0;	// matrix number being initialized

	constraints = new GridBagConstraints();
	constraints.weightx = 0;
	constraints.weighty = 0;
	constraints.gridheight = 1;
	constraints.gridwidth  = 1;
	constraints.anchor = constraints.NORTHWEST;

	for(int x = 0; x < size ; x++){ // initalize the textFields
	    for(int y = 1; y < size + 1; y++){
		constraints.gridx = y;
		constraints.gridy = x + 1;
		memoryMatrixPanel.add(Globals.matrixFields[count++], constraints);
	    }
	}
    }// end initMatrix()

    public void setColors(){
	int size = (int)Math.sqrt(Globals.MEMSIZE);	// for matrix labels

	// set the color for the program counter
	Globals.programCounterField.setForeground(OurColors.programCounterFieldFore);
	Globals.programCounterField.setBackground(OurColors.programCounterFieldBack);
	// set the colors for the registers
	for (int i = 0; i < Globals.REGSIZE;  i++){
		Globals.registerLabels[i].setForeground(OurColors.registerLabelsFore);
		Globals.registerLabels[i].setBackground(OurColors.registerLabelsBack);
		//Globals.registerOutputLabels[i].setForeground(OurColors.registerOutputLabelsFore);
		Globals.registerOutputLabels[i].setForeground(OurColors.registerOutputLabelsFore);
		Globals.registerOutputLabels[i].setBackground(OurColors.registerOutputLabelsBack);
	    }
	// set the colors for the memory list
	for (int i = 0; i < Globals.MEMSIZE / 2; i++){
		Globals.memoryAddressLabels[i].setForeground(OurColors.memoryAddressLabelsFore);
		Globals.memoryAddressLabels[i].setBackground(OurColors.memoryAddressLabelsBack);
		Globals.memoryInputFields[i].setForeground(OurColors.inputFieldsFore);
		Globals.memoryInputFields[i].setBackground(OurColors.inputFieldsBack);
		Globals.outputLabels[i].setForeground(OurColors.outputLabelsFore);
		Globals.outputLabels[i].setBackground(OurColors.outputLabelsBack);
	    }
	// set the colors for the matrix
	for (int i = 0; i < size; i++){
		Globals.topMatrixLabels[i].setForeground(OurColors.memoryAddressOutputLabelsFore);
		Globals.topMatrixLabels[i].setBackground(OurColors.memoryAddressOutputLabelsBack);
		Globals.sideMatrixLabels[i].setForeground(OurColors.memoryAddressOutputLabelsFore);
		Globals.sideMatrixLabels[i].setBackground(OurColors.memoryAddressOutputLabelsBack);
	    }
	// set the color of the matrix textfields
	for (int i = 0; i < Globals.MEMSIZE; i++){
		Globals.matrixFields[i].setForeground(OurColors.inputFieldsFore);
		Globals.matrixFields[i].setBackground(OurColors.inputFieldsBack);
	    }

	// set the colors of the various titles
	registerPanelLabel.setForeground(OurColors.registerPanelLabel);
	programCounterTitleLabela.setForeground(OurColors.programCounterTitleLabel);
	programCounterTitleLabelb.setForeground(OurColors.programCounterTitleLabel);
	memoryPanelLabel.setForeground(OurColors.memoryPanelLabel);

	// set outer panel backgrounds
	westPanel.setBackground(OurColors.outerBackground);
	centerPanel.setBackground(OurColors.outerBackground);
	registerPanel.setBackground(OurColors.outerBackground);
	buttonPanel.setBackground(OurColors.outerBackground);
	counterPanel.setBackground(OurColors.outerBackground);
	outputPanel.setBackground(OurColors.outerBackground);

	// set inner panel backgrounds
	memoryListPanel.setBackground(OurColors.innerBackground);
	memoryMatrixPanel.setBackground(OurColors.innerBackground);

	Globals.scrollPanePanel.setBackground(OurColors.innerBackground);

	// set the bottom output textfield colors
	Globals.bottomOutputField.setBackground(OurColors.bottomOutputFieldBack);
	Globals.bottomOutputField.setForeground(OurColors.bottomOutputFieldFore);

	// set button colors
	run.setForeground(OurColors.buttonFore);
	run.setBackground(OurColors.buttonBack);
	stop.setForeground(OurColors.buttonFore);
	stop.setBackground(OurColors.buttonBack);
	step.setForeground(OurColors.buttonFore);
	step.setBackground(OurColors.buttonBack);
	reset.setForeground(OurColors.buttonFore);
	reset.setBackground(OurColors.buttonBack);
    }

    private void getPreferences(){
	prefDialog.displayDialog();
    }// end getPreferences()

    void newSpeed(int val){
	Globals.sleepTime = 1000-val;
	if(Globals.sleepTime > 1000){
	    Globals.sleepTime = 1000;
	}

	if(Globals.sleepTime < 0){
	    Globals.sleepTime = 0;
	}
	//delayField.setText(""+Globals.sleepTime);

    }// end newSpeed();


    public void addActionStuff(){
	//viewType.addItemListener( itemHandler );
	//helpHelp.addItemListener( itemHandler );
	speedSlider.addAdjustmentListener(adjustmentHandler);

	fileOpen.addActionListener( actionHandler );
	fileSave.addActionListener( actionHandler );
	fileClear.addActionListener( actionHandler );
	fileExit.addActionListener( actionHandler );


	editUndo.addActionListener( actionHandler );
	editInsert.addActionListener( actionHandler );
	editDelete.addActionListener( actionHandler );
	editPreferences.addActionListener( actionHandler );

	helpCommands.addActionListener( actionHandler );
	helpInfo.addActionListener( actionHandler );
	helpAbout.addActionListener( actionHandler );
	helpClose.addActionListener( actionHandler );

	run.addActionListener( actionHandler );
	stop.addActionListener( actionHandler );
	step.addActionListener( actionHandler );
	reset.addActionListener( actionHandler );


	for(int x = 0; x < Globals.MEMSIZE/2; x++){
	    Globals.memoryInputFields[x].addKeyListener(new KeyGrabber(x));
	    Globals.memoryInputFields[x].addFocusListener(new FocusGrabber(x));
	}

	for(int x = 0; x< Globals.MEMSIZE; x++){
	    Globals.matrixFields[x].addFocusListener(new FocusGrabber(x));
	}
	Globals.memoryInputFields[0].requestFocus();

    }// end addActionStuff()


    //***********************************************
    //
    //	Inner Classes for Action grabbing are here.
    //
    //
    //

    class ActionGrabber implements ActionListener{

	public void actionPerformed(ActionEvent e){
	    String command = e.getActionCommand();

	    if( command.equals("Import/Export")){
		System.out.println("File | Import/Export selected");
		Commands.open();

	    }else if( command.equals("Email Code")){

		Globals.saveWindow.setEmail(Globals.lastEmail);
		Commands.save();
		System.out.println("File | Email Code selected");

	    }else if( command.equals("Clear")){

		System.out.println("File | Clear selected");
		Commands.clear();

	    }else if( command.equals("Exit")){

		System.out.println("File | Exit selected");
		Commands.exit();

	    }else if( command.equals("Insert")){
		if (Globals.shiftingOn){
		    System.out.println("Edit | Shift Insert selected");
		    Commands.shiftInsert();
		}else{
		    System.out.println("Edit | Insert selected");
		    Commands.insert();
		}
	    
	    }else if( command.equals("Delete")){
		if (Globals.shiftingOn){
		    System.out.println("Edit | Shift Delete selected");
		    Commands.shiftDelete();
		}else{
		    System.out.println("Edit | Delete selected");
		    Commands.delete();
		}
	    }else if( command.equals("Undo Insert/Delete")){

		System.out.println("Edit | Undo selected");
		Commands.undo(true);
	    }else if( command.equals("Preferences")){
		System.out.println("Edit | Preferences selected");
		getPreferences();
	    }else if( command.equals("Commands")){
		
		System.out.println("Help | Commands selected");
		helpSys.show("Commands");
		helpClose.setEnabled(true);
		System.out.println("Globals.helpShowing ="+Globals.helpShowing);
		
	    }else if( command.equals("Compiler Info")){

		System.out.println("Help | Compiler Info selected");
		helpSys.show("Compiler");
		helpClose.setEnabled(true);
		System.out.println("Globals.helpShowing ="+Globals.helpShowing);
	    }else if( command.equals("About")){

		System.out.println("Help | About selected");
		helpSys.show("About");
		helpClose.setEnabled(true);
		System.out.println("Globals.helpShowing ="+Globals.helpShowing);

	    }else if( command.equals("Close Help")){
		
		System.out.println("Help | Close Help selected");
		if(Globals.helpShowing){
		    helpSys.showHelp();
		    helpClose.setEnabled(false);
		}
		System.out.println("Globals.helpShowing ="+Globals.helpShowing);
		
	    }else if( command.equals("Run")){
		
		System.out.println("Run pressed");
		Commands.run();

	    }else if( command.equals("Halt")){
		
		System.out.println("Halt pressed");
		Commands.stop();
		
	    }else if( command.equals("Step")){
		
		System.out.println("Step pressed");

		if (Globals.programRunner != null){
			
		    Globals.programRunner = null;
		    Globals.isThreaded = false;
		    Globals.mainGUI.run.setEnabled(true);
		}else{
		    Commands.step();
		}
	    
	    }else if( command.equals("Reset Registers/PC")){
		System.out.println("Reset Registers/PC pressed");
		Commands.reset();
	    }
	}// end actionPerformed()
    }// end inner class actionGrabber

    class KeyGrabber implements KeyListener
    {
	int index;

	public KeyGrabber(int index)
	{
	    super();
	    this.index = index;
	}

	public KeyGrabber()
	{
	    super();
	}

	public void keyTyped(KeyEvent e)
	{
	    System.out.println("KeyTyped: "+e);
	}// end keyTyped()

	public void keyReleased(KeyEvent e)
	{
	    System.out.println("Component: "+e.getComponent());

	    //if((e.getKeyChar() >= 32) && (e.getKeyChar() <= 126))
	    {
		System.out.println("Visible key pressed, event:" +e);
		System.out.println(((TextField)e.getComponent()).getText());
		//if(((TextField)e.getComponent()).getText().length() >= 4 )
		{
		    Commands.getInput(index);
		}
	    }

	}// end keyReleased()

	public void keyPressed(KeyEvent e)
	{
	    System.out.println("KeyPressed: "+e);
	    if(e.getKeyCode() == 9)
		{
		    Commands.getInput(index);
		}
	}// end keyPressed()

    }// end class KeyGrabber

    class FocusGrabber implements FocusListener
    {
	int index;

	public FocusGrabber(int index)
	{
	    this.index = index;
	}// end FocusGrabber()

	public void focusGained(FocusEvent e)
	{
	    if(Globals.matrixShowing)
		{
		    return;
		}
	    int yloc = (int)(Globals.memoryInputFields[index].getLocation().y - memoryListScrollPane.getScrollPosition().y);
	    //yloc is the position relative to the top of the scrollPane.
	    int viewportSize = (int)memoryListScrollPane.getViewportSize().height;

	    Globals.hasFocus = index;

	    if((memoryListScrollPane.getScrollPosition().y > 56) && (memoryListScrollPane.getScrollPosition().y + viewportSize < memoryListPanel.getBounds().height));
	    {
		if(yloc > viewportSize-56)
		    {
			memoryListScrollPane.setScrollPosition(0,(int)(memoryListScrollPane.getScrollPosition().y+23));
		    }
		if(yloc < 56)
		    {
			memoryListScrollPane.setScrollPosition(0,(int)(memoryListScrollPane.getScrollPosition().y-23));
		    }
	    }

	}// end focusGained()

	public void focusLost(FocusEvent e)
	{

	}// end focusLost()

    }// end class FocusGrabber

    class MouseGrabber implements MouseListener
    {
	int index;

	public MouseGrabber(int index)
	{
	    super();
	    this.index = index;
	}// end MouseGrabber()

	public MouseGrabber()
	{

	}// end MouseGrabber()

	public void mouseClicked(MouseEvent e)
	{

	}// end mouseClicker()

	public void mousePressed(MouseEvent e)
	{

	}// end mousePressed()

	public void mouseReleased(MouseEvent e)
	{

	}// end mouseReleased()

	public void mouseEntered(MouseEvent e)
	{

	}// end mouseEntered()

	public void mouseExited(MouseEvent e)
	{

	}// end mouseExited()

    }// end class MouseGrabber

    class AdjustmentGrabber implements AdjustmentListener
    {
	public void adjustmentValueChanged(AdjustmentEvent e)
	{
	    newSpeed(e.getValue()*8);
	}// end adjustmentValueChanged()
    }// end class AdjustmentGrabber

}// end class CompilerGUI
