//     The Brookshear Machine
//     Copyright (C) 2004 E. Rogan Creswick
//
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

/**
*
*		CompilerGUI
* Last edited: 1/29/2000, 10:25 PM by Rogan
*
* Revision history:
*
*	1/29/00 - file created.
*
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*		Description:  FieldLabel is a label
*	which can be changed by the user simply
*	by clicking on it.
**/


import java.awt.*;
import java.awt.event.*;

public class FieldLabel extends Container{

	String text;			// the text that is displayed on the component
	String currentState;	// either field or label

	Label		thisLabel = new Label();
	TextField 	thisField;					// textField will be instantiated when it is needed.


	CardLayout mainLayout = new CardLayout();
	Color foreground,background;

	public FieldLabel(String text)	// constructor
	{
		setText(text);

		initWidgits();

	}// end FieldLabel()


	private void initWidgits()
	{
		this.setLayout(mainLayout);
		setText(this.text);

		initLabel();
	}// end initWidgits()

	private void initLabel()
	{
		add("thisLabel", thisLabel);
		thisLabel.addMouseListener(new MouseGrabber());
	}// end initLabel()

	private void initTextField()
	{
		thisField = new TextField(this.text); // creates a text field with current text.
		add("thisField", thisField);
		thisField.addFocusListener( new FocusGrabber());	// the FocusGrabber will change this back to
															// a label when it looses focus.
		thisField.addKeyListener( new KeyGrabber());		// changes back to label on carrige return
	}// end createTextField()


	private void showTextField()
	{
		initTextField();
		setText(thisLabel.getText());
		thisField.selectAll();
		thisField.requestFocus();

		currentState = "textField";
		mainLayout.last(this);
	}// end showTextField()

	private void showLabel()
	{
		setText(thisField.getText());
		currentState = "label";
		mainLayout.first(this);
	}// end showLabel()

	//***********************************
	//  accessor / mutator methods are
	//	declared here.
	//

	public void setBackground(Color background)
	{
		this.background = background;
	}// end setBackground()

	public void setForeground(Color foreground)
	{
		this.foreground = foreground;
	}// end setForeground()

	public void setText(String text)
	{
		this.text = text;
		thisLabel.setText(text);

		if(thisField != null)			// not all textFields are instantiated,
			thisField.setText(text);	// this is done to save memory.
	}// end setText()

	public String toString()
	{
		return "FieldLabel: state: "+currentState+" Text: "+text;
	}// end toString()


	//***********************************
	//	Inner classes for action handling
	//	are here.
	//

	class MouseGrabber implements MouseListener
	{
		public void mouseClicked(MouseEvent e)
		{
			showTextField();
		}// end mouseClicker()

		public void mousePressed(MouseEvent e)
		{

		}// end mousePressed()

		public void mouseReleased(MouseEvent e)
		{

		}// end mouseReleased()

		public void mouseEntered(MouseEvent e)
		{

		}// end mouseEntered()

		public void mouseExited(MouseEvent e)
		{

		}// end mouseExited()

	}// end class MouseGrabber

	class FocusGrabber implements FocusListener
	{
		public void focusGained(FocusEvent e)
		{

		}// end focusGained()

		public void focusLost(FocusEvent e) // displays label again.
		{
			showLabel();
		}// end focusLost()

	}// end class FocusGrabber

	class KeyGrabber implements KeyListener
	{

		public void keyTyped(KeyEvent e)
		{

		}// end keyTyped()

		public void keyReleased(KeyEvent e)
		{

		}// end keyReleased()

		public void keyPressed(KeyEvent e)
		{
			if(e.getKeyCode() == 10)
			{
				showLabel();
			}
		}// end keyPressed()

	}// end class KeyGrabber


}// end class FieldLabel