//     The Brookshear Machine
//     Copyright (C) 2004   E. Rogan Creswick

//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.

//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/**
Globals.java
*
* Last edited on 2/14/00 at 1:38 AM by Stacy Brock
*
* Revision history:
*
* 1/21/00 - Reorganized variables
*			added toInt(String n) to convert strings of hex to int
* 1/22/00 - added errorPrompt(String msg) for confirmation of critical
*			actions.
* 1/31/00 - Moved updateMemoryAddresses() from operators to this class
*			elimanted outputtext[] array -GH
* 2/3/00  - Added aboutWindow
* 2/6/00  - Changed register and memory address values to bytes
* 2/14/00 - MemoryBackup is now a multidimensional array, added int
*           CurrentUndoPosition, added final int UNDOLEVELS
* 6/18/04 ERC  Re-formatted the code. Added some comments.
**/


import java.awt.*;
import java.awt.event.*;
import java.net.*;

/**
 *  @class Globals
 *
 *  Contains all global variables for the compiler applet.  This
 *  design was chosen because the scope of the project was small and
 *  it was easier than dealing with communication between classes.
 *  Frankly, we didn't know a better way to do it, and this works
 *  reasonably well for this project.  Although, there may be
 *  synchronization bugs (I don't think there are, because the
 *  interface is locked during the run phase, but I'm not 100%
 *  certain.)
 *
 */
public class Globals{
    
    /**
     *  The applet object, some aspects (the mailer, I believe) need to access this object.
     */
    static CompilerApplet applet;

    /**
     *  
     */
    static Frame rootObj;

    /**
     * 
     */
    static URL codeBaseURL;
    
    /**
     *  The default delay for the timer.  Stored as a global so both
     *  threads can see it (but only one modifies it).  This is a
     *  potential source of bugs, although not critical ones.  (see
     *  the readers & writers problem)
     */
    static int sleepTime = 500;

    /**
     *  The thread that actually runs the program that the user has
     *  entered.  This thread does all the computations, moves memory,
     *  highlights instructions, etc.
     */
    static CompilerThread programRunner;
    
    /**
     *
     */
    public static Commands command = new Commands();
    
    /**
     *  This is the primary window that appears (with the registers, pc, etc..)
     */
    public static CompilerGUI mainGUI;
    
    public static Mailer mail;
    public static SaveDialog saveWindow;
    public static IODialog openWindow;
    
    /**
     *  The window title. Originally, this was "Compiler"
     */
    public static final String TITLE = "The Brookshear Machine";
    public static final int MEMSIZE = 256;
    public static final int REGSIZE = 16;
    public static final int UNDOLEVELS = 10;
    
    //public static Panel scrollPanePanel   = new Panel(); // Panel containing the CardLayout.
    public static TabPanel scrollPanePanel  = new TabPanel();
    
    public static byte MemoryAddress[]				= new byte[MEMSIZE * 2];
    public static byte[][] MemoryBackup				= new byte[UNDOLEVELS][MEMSIZE * 2];
    public static byte RegisterAddress[]			= new byte[REGSIZE * 2];
    public static int CurrentUndoPosition = -1;
    public static int ProgramCounter;
    public static boolean stopProgram = false;
    public static boolean isRunning = false;
    public static boolean isThreaded = false;
    public static boolean shiftingOn = true;
    
    public static TextField matrixFields[]			= new TextField[MEMSIZE];
    public static TextField memoryInputFields[]		= new TextField[128];
    
    public static TextField programCounterField		= new TextField("00",2);
    
    public static Label registerLabels[]			= new Label[REGSIZE];
    //public static Label registerOutputLabels[]		= new Label[REGSIZE];
    public static TextField registerOutputLabels[]	= new TextField[REGSIZE];
    public static Label topMatrixLabels[]			= new Label[(int)Math.sqrt(MEMSIZE)];
    public static Label sideMatrixLabels[]			= new Label[(int)Math.sqrt(MEMSIZE)];
    public static Label memoryAddressLabels[]		= new Label[MEMSIZE/2];
    //public static Label memoryAddressOutputLabels[]	= new Label[MEMSIZE/2];
    public static FieldLabel outputLabels[]			= new FieldLabel[MEMSIZE/2];
    
    public static TextField bottomOutputField	= new TextField();
    
    public static boolean helpShowing = false;
    public static boolean matrixShowing = false;
    public static String lastEmail;
    
    public static int lineCount = 0;
    
    public static boolean errorPromptBool;
    
    public static int hasFocus;
    
    //*************************
    // These boolean variables are for the PrefrencesDialog class.
    public static boolean haltOnErrors  = false;
    public static boolean shiftOn 	= true;
    public static boolean backupCode 	= false;
    // End PrefrencesDialog variables.
    //*************************
    
    
    public static String findCode(){
    
	Commands.getInput();
	String code = new String("--EndCode--");
	String temp = "";
	boolean tooLong = true;

	for(int i = MEMSIZE/2-1; i >= 0; i--){
	    if (tooLong){
		if(MemoryAddress[4 * i] + MemoryAddress[4 * i + 1]
		   + MemoryAddress[4 * i + 2] + MemoryAddress[4 * i + 3] != -4){
		    tooLong = false;
		}
	    }
	    if (!tooLong){
		if(MemoryAddress[4 * i] + MemoryAddress[4 * i + 1]
		   + MemoryAddress[4 * i + 2] + MemoryAddress[4 * i + 3] == -4){
		    temp = "";
		}else{
		    if(MemoryAddress[4 * i] != -1)
			temp = toHex(MemoryAddress[4 * i]);
		    else
			temp = " ";
		    if(MemoryAddress[4 * i + 1] != -1)
			temp += toHex(MemoryAddress[4 * i + 1]);
		    else
			temp += " ";
		    if(MemoryAddress[4 * i + 2] != -1)
			temp += toHex(MemoryAddress[4 * i + 2]);
		    else
			temp += " ";
		    if(MemoryAddress[4 * i + 3] != -1)
			temp += toHex(MemoryAddress[4 * i + 3]);
		    else
			temp += " ";
		}
		code = temp + "\n" + code;
	    }
	}

	return  "--Code--\n" + code;
    }// end findCode()


    /**
     *  Displays a modal error dialog with the given message, one
     *  button "Ok" is available to dismiss the dialog.
     *
     *  @param errorMsg The error message to be displayed.
     */
    public static void errorDialog(String errorMsg){
	
	int HEIGHT = 150;
	int WIDTH = 250;
	
	final Dialog err = new Dialog(new Frame(),"Error", true);
	Label msg = new Label(errorMsg, 1); // sets alignment to center
	msg.setForeground(OurColors.errorText);
	
	Panel buttonPanel = new Panel(new FlowLayout(FlowLayout.CENTER, 10,10));
	Button ok = new Button("Ok");
	
	ok.setBackground(OurColors.buttonFore);
	ok.setBackground(OurColors.buttonBack);
	err.setBackground(OurColors.outer);
	err.setBounds((mainGUI.rootFrame.getLocationOnScreen().x-(WIDTH/2 - mainGUI.rootFrame.getBounds().width/2)), 
		      (mainGUI.rootFrame.getLocationOnScreen().y+((mainGUI.rootFrame.getBounds().height/2)-HEIGHT/2)),
		      WIDTH, HEIGHT);
	err.setResizable(true);
	
	
	ok.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e){
		    if((e.getActionCommand()).equals("Ok")){
			err.dispose();
		    }
		}
	    });// end inner-annonomous actionListener class

	err.setLayout(new GridLayout(2,1));
	err.add(msg);
	buttonPanel.add(ok);
	err.add(buttonPanel);
	
	err.show();
    }// end errorDialog(String)
    
    /**
     *  Presents errorMsg in a modal error dialog with "Stop" and "continue" buttons.
     *
     *   @param errorMsg The string to display in the error dialog.
     */
    public static void errorPrompt(String errorMsg){
	int HEIGHT = 150;
	int WIDTH = 250;
	
	final Dialog err = new Dialog(new Frame(),"Error", true);
	Label msg = new Label(errorMsg, 1); // sets alignment to center
	msg.setForeground(OurColors.errorText);
	Panel buttonPanel = new Panel(new FlowLayout(FlowLayout.CENTER, 10,10));
	Button cont = new Button("Continue");
	Button stop = new Button("Cancel");
	
	stop.setBackground(OurColors.buttonFore);
	stop.setBackground(OurColors.buttonBack);
	cont.setBackground(OurColors.buttonFore);
	cont.setBackground(OurColors.buttonBack);
	err.setBackground(OurColors.outer);
	err.setBounds((mainGUI.rootFrame.getLocationOnScreen().x-(WIDTH/2 - mainGUI.rootFrame.getBounds().width/2)), (mainGUI.rootFrame.getLocationOnScreen().y+((mainGUI.rootFrame.getBounds().height/2)-HEIGHT/2)), WIDTH, HEIGHT);
	err.setResizable(false);
	
	
	cont.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e){
		    if((e.getActionCommand()).equals("Continue")){
			errorPromptBool = true;
			err.dispose();
		    }
		}
	    });// end inner-annonomous actionListener class
	
	stop.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e){
		    if((e.getActionCommand()).equals("Cancel")){
			errorPromptBool = false;
			err.dispose();
		    }
		}
	    });// end inner-annonomous actionListener class

	err.setLayout(new GridLayout(2,1));
	err.add(msg);
	buttonPanel.add(cont);
	buttonPanel.add(stop);
	err.add(buttonPanel);
	
	err.show();
	
    }// end errorPrompt(String )

    public static void updateTextfields(){
	
	System.out.println("updateTextfields() called");
	String temp;
	if (!matrixShowing){
	    for (int i = 0; i < MEMSIZE / 2; i++){
		if(MemoryAddress[4*i] + MemoryAddress[4*i + 1]
		   + MemoryAddress[4*i + 2] + MemoryAddress[4*i + 3] == -4){
		    memoryInputFields[i].setText("");
		    outputLabels[i].setText("");
		}else{
		    if(MemoryAddress[4 * i] != -1)
			temp = toHex(MemoryAddress[4 * i]);
		    else
			temp = "0";
		    if(MemoryAddress[4 * i + 1] != -1)
			temp += toHex(MemoryAddress[4 * i + 1]);
		    else
			temp += "0";
		    if(MemoryAddress[4 * i + 2] != -1)
			temp += toHex(MemoryAddress[4 * i + 2]);
		    else
			temp += "0";
		    if(MemoryAddress[4 * i + 3] != -1)
			temp += toHex(MemoryAddress[4 * i + 3]);
		    else
			temp += "0";
		    
		    memoryInputFields[i].setText(temp);
		    Output.displayOutputText(i);
		}
	    }
	}else{
	    for (int i = 0; i < MEMSIZE; i++){
		if (MemoryAddress[2*i] + MemoryAddress[2*i + 1] == -2){
		    matrixFields[i].setText("");
		}else{
		    if(MemoryAddress[2 * i] != -1)
			temp = toHex(MemoryAddress[2 * i]);
		    else
			temp = "0";
		    if(MemoryAddress[2 * i + 1] != -1)
			temp += toHex(MemoryAddress[2 * i + 1]);
		    else
			temp += "0";
		    matrixFields[i].setText(temp);
		}
	    }
	}
    }//end updateTextfields()

    public static void updateTextfields(int location){
	//System.out.println("updateTextfields(int) location: "+location);

	String newFrontByte = "";
	String newBackByte = "";
	// get the bytes from the memory array
	if(MemoryAddress[4*location] + MemoryAddress[4 * location + 1] != -2){
	    newFrontByte = ("" + toHex(MemoryAddress[4*location])
			    + toHex(MemoryAddress[4*location + 1]));
	}
	if(MemoryAddress[4*location + 2] + MemoryAddress[4 * location + 3] != -2){
	    newBackByte = ("" + toHex(MemoryAddress[4*location + 2])
			   + toHex(MemoryAddress[4*location + 3]));
	}
	if(MemoryAddress[4*location] + MemoryAddress[4 * location + 1] 
	   + MemoryAddress[4*location + 2] + MemoryAddress[4 * location + 3] == -4){
	    newFrontByte = "";
	    newBackByte = "";
	    outputLabels[location].setText("");
	}
	
	// set the text fields:
	if (!matrixShowing){
	    memoryInputFields[location].setText(newFrontByte+newBackByte);
	    Output.displayOutputText(location);
	}else{
	    matrixFields[2 * location].setText(newFrontByte);
	    matrixFields[2 * location + 1].setText(newBackByte);
	}

    }// end updateTextfields(int )

    public static String toHex(int n){
	if (n == -1)
	    return "-1";
	if( n <= 9){
	    return "" + n;
	}else if( n > 9 && n < 16){
	    return "" + (char) (n + 87);
	}else{
	    return  toHex(n >> 4) + toHex(n % 16);
	}
    }// end toHex()

    public static byte toInt(char n){
	if(n >= 48 && n <= 57){
	    return (byte)(n - 48);
	}else if(n >= 65 && n <=70 ){
	    return (byte)(n - 55);
	}else if(n >= 97 && n <= 102 ){
	    return (byte)(n - 87);
	}else{
	    System.out.println("Invalid value, input must fit in 0 <= input <= f");
	    return -1;
	}
    }// end toInt()

    public static int toInt(String n){
	int returnInt = 0;
	System.out.println("toInt("+n+")");
	
	int count = n.length() - 1;
	
	returnInt += toInt(n.charAt(count));
	
	for(; count >= 0; count--){
	    returnInt += (((n.length() - count) - 1) * 16) * toInt(n.charAt(count));
	}
	System.out.println("toInt return:"+returnInt);
	return returnInt;

    }// end toInt(String n)
}// end class Globals
