/** MailIO.java
 * Last edited on 1/23/00 at 11:32 PM by Stacy Brock
 *
 * CompilerIO takes input to be saved and sends it via e-mail
 * to a destination specified by the user.
 *
 * Revision history:
 * 1/20/00 - file created
 * 1/23/00 - changed PrintStream to BufferedWriter/OutputStreamWriter
 *         - send() actually sends real input, not foo
 *         - added getTimeString() method to calculate current time/date
 *
 **/

import java.io.*;
import java.net.*;
import java.util.*;


public class MailIO {
	static final String EOL = "\r\n"; // network end of line
	GregorianCalendar gc;
	Date d;

	String mailhost;
	//String mailhost = "128.193.4.11";// bobo.nws.orst.edu
	//String mailhost = "128.193.54.1"; // engr.orst.edu
	int port = 25;
	String lhIP;
	InetAddress localhost;
	String sender = "";		// From: e-mail addy
	String subject = "Applet Test Message";
	String recipient = "brock@engr.orst.edu";	// To: e-mail addy
	String message = "This is a test message. Please disregard.";

	BufferedWriter send = null;
	BufferedReader reply = null;
	Socket sock = null;

	public MailIO (String _sender) {
		this.sender = _sender;
		this.subject = subject;
		this.recipient = recipient;
		this.message = message;
		gc = new GregorianCalendar(TimeZone.getDefault());
		//System.out.println("Time: " + gc.getTime());
		//System.out.println("Time String: " + getTimeString());
		mailhost = Globals.codeBaseURL.getHost();
		System.out.println("mailhost = "+mailhost);

		try {
			sock = new Socket(mailhost, port);
			reply = new BufferedReader(new InputStreamReader(sock.getInputStream()));
			//send = new PrintStream(sock.getOutputStream());
			send = new BufferedWriter(new OutputStreamWriter(sock.getOutputStream()));
		} catch (Exception e) {
			System.out.println(e);
		}

	} // end constructor MailIO

	public void send(String _recipient, String _subject, String _message) throws IOException {
		System.out.println("send() method called...");
		localhost = null;
		String rString, sString;
		recipient = _recipient;
		subject = _subject;
		message = _message;
		try {
			localhost = InetAddress.getLocalHost();
			lhIP = localhost.getHostName();
			System.out.println("localhost is: " + localhost);
		} catch (UnknownHostException ioe) {
			System.out.println("No local IP address found.");
		}

		rString = reply.readLine();
		System.out.println(rString);
		if (!rString.startsWith("220")) throw new ProtocolException(rString);
		quicksend("EHLO " + lhIP);
		for(int i = 0; i < 8; i++) {
			rString = reply.readLine();
			System.out.println(rString);
			if (!rString.startsWith("250")) throw new ProtocolException(rString);
		}
		quicksend("RSET");
		rString = reply.readLine();
		System.out.println(rString);
		if (!rString.startsWith("250")) throw new ProtocolException(rString);
		quicksend("MAIL FROM:<" + sender + ">");
		rString = reply.readLine();
		System.out.println(rString);
		if (!rString.startsWith("250")) throw new ProtocolException(rString);
		quicksend("RCPT TO:<" + recipient + ">");
		rString = reply.readLine();
		System.out.println(rString);
		if (!rString.startsWith("250")) throw new ProtocolException(rString);
		quicksend("DATA");
		rString = reply.readLine();
		System.out.println(rString);
		if (!rString.startsWith("354")) throw new ProtocolException(rString);

		quicksend("X-Sender: CS160 Compiler Applet");
		quicksend("X-Mailer: CS160 Compiler Applet");
		quicksend("Date: Sun, 23 Jan 2000 18:22:00 -8000");
		//quicksend("Date: " + getTimeString());
		quicksend("To: " + recipient);
		quicksend("From: CS160 Compiler Applet <" + sender + ">");
		quicksend("Subject: " + subject);
		quicksend("Mime-Version: 1.0");
		quicksend("Content-Type: text/plain; charset=\"us-ascii\"; format=flowed");
		quicksend(EOL);
		quicksend(message);
		quicksend(EOL);
		quicksend(".");
		quicksend(EOL);
		rString = reply.readLine();
		System.out.println(rString);
		if (!rString.startsWith("250")) throw new ProtocolException(rString);
		quicksend("QUIT");
		rString = reply.readLine();
		System.out.println(rString);
		if (!rString.startsWith("221")) throw new ProtocolException(rString);

	} // end method send()

	public void quicksend(String m) {
		try {
			System.out.println(m);
			send.write(m, 0, m.length());
			if (!m.equals(EOL)) send.write(EOL, 0, EOL.length());
			send.flush();
		} catch (IOException ioe) {
			System.out.println(ioe);
		}
	} // end method quicksend()

/*	public String getTimeString() {
		String s = "";
		//System.out.println(gc.get(gc.DAY_OF_WEEK));
		switch(gc.get(gc.DAY_OF_WEEK)) {
			case 1: s = s + "Sun, ";
					break;
			case 2: s = s + "Mon, ";
					break;
			case 3: s = s + "Tue, ";
					break;
			case 4: s = s + "Wed, ";
					break;
			case 5: s = s + "Thu, ";
					break;
			case 6: s = s + "Fri, ";
					break;
			case 7: s = s + "Sat, ";
					break;
		}
		//System.out.println(gc.get(gc.DAY_OF_MONTH));
		s = s + gc.get(gc.DAY_OF_MONTH) + " ";
		//System.out.println(gc.get(gc.MONTH));
		switch(gc.get(gc.MONTH)) {
			case 0: s = s + "Jan ";
					break;
			case 1: s = s + "Feb ";
					break;
			case 2: s = s + "Mar ";
					break;
			case 3: s = s + "Apr ";
					break;
			case 4: s = s + "May ";
					break;
			case 5: s = s + "Jun ";
					break;
			case 6: s = s + "Jul ";
					break;
			case 7: s = s + "Aug ";
					break;
			case 8: s = s + "Sep ";
					break;
			case 9: s = s + "Oct ";
					break;
			case 10: s = s + "Nov ";
					break;
			case 11: s = s + "Dec ";
					break;
		}
		s = s + gc.get(gc.YEAR) + " ";
		s = s + gc.get(gc.HOUR_OF_DAY) + ":";
		s = s + gc.get(gc.MINUTE) + ":";
		s = s + gc.get(gc.SECOND) + " ";
		//System.out.println(gc.getTimeZone().getDisplayName());
		if(gc.getTimeZone().getDisplayName().equals("Pacific Standard Time")) {
			s = s + "-8000";
		} else if(gc.getTimeZone().getDisplayName().equals("Mountain Standard Time")) {
			s = s + "-7000";
		} else if(gc.getTimeZone().getDisplayName().equals("Central Standard Time")) {
			s = s + "-6000";
		} else if(gc.getTimeZone().getDisplayName().equals("Eastern Standard Time")) {
			s = s + "-5000";
		}
		return s;
	} // end method getTimeString()
*/
} // end class MailIO