/**
*
*		Operators
* Last edited: 2/13 by Gary
*
* Revision history:
*
* 1/21/00 - Fixed Floating point operations
* 1/22/00 - Fixed updateMemoryAddress to correctly update
*
* 2/13 - Added error detection/correction
*
**/

public class Operators
{
    private static void updateRegisterAddresses(int location)
    {
	Globals.registerOutputLabels[location].setText(
						       Globals.toHex(Globals.RegisterAddress[2 * location])
						       + Globals.toHex(Globals.RegisterAddress[2 * location + 1]) );
    }

    private static void updateProgramCounter()
    {
	if(Globals.ProgramCounter < 16)
	    {
		Globals.programCounterField.setText("0"+Globals.toHex(Globals.ProgramCounter));
	    }
	else
	    {
		Globals.programCounterField.setText(Globals.toHex(Globals.ProgramCounter));
	    }
    }

    private static int temp, temp2, temp3;
    private static byte instruction1, instruction2, instruction3, instruction4;
    private static char outchar2, outchar3, outchar4;
    private static boolean noValue, stopProgram, warningError;
    private static String outputString;

    public static void ProcessOperator(int MemoryLocation)
    {
	stopProgram = false;
	noValue = false;
	outputString = "";
	instruction1 = Globals.MemoryAddress[MemoryLocation * 4];
	instruction2 = Globals.MemoryAddress[MemoryLocation * 4 + 1];
	instruction3 = Globals.MemoryAddress[MemoryLocation * 4 + 2];
	instruction4 = Globals.MemoryAddress[MemoryLocation * 4 + 3];

	outchar2 = Globals.toHex(Globals.MemoryAddress[MemoryLocation * 4 + 1]).charAt(0);
	outchar3 = Globals.toHex(Globals.MemoryAddress[MemoryLocation * 4 + 2]).charAt(0);
	outchar4 = Globals.toHex(Globals.MemoryAddress[MemoryLocation * 4 + 3]).charAt(0);
	if (instruction1 == -1)
	    {
		instruction1 = 0;
		noValue = true;
	    }
	if (instruction2 == -1)
	    {
		instruction2 = 0;
		noValue = true;
	    }
	if (instruction3 == -1)
	    {
		instruction3 = 0;
		noValue = true;
	    }
	if (instruction4 == -1)
	    {
		instruction4 = 0;
		noValue = true;
	    }
	Globals.ProgramCounter += 2;
	updateProgramCounter();
	Globals.bottomOutputField.setForeground(OurColors.bottomOutputFieldFore);

	System.out.println("instruction1:" + instruction1);

	switch(instruction1)
	    {
	    case 1:		LoadAddress();
		break;
	    case 2:		Load();
		break;
	    case 3:		Store();
		break;
	    case 4:		Move();
		break;
	    case 5:		Add();
		break;
	    case 6:		AddFloat();
		break;
	    case 7:		Or();
		break;
	    case 8:		And();
		break;
	    case 9:		ExclusiveOr();
		break;
	    case 10:	Rotate();
		break;
	    case 11:	Jump();
		break;
	    case 12:	Halt();
		break;
	    case 13:    JumpGreater();
		break;
	    default:	noOpCode();
		break;
	    }
	if (noValue)
	    {
		Globals.stopProgram = true;
		outputString = "WARNING: Missing value at memory location, value defaulting to 0: " + outputString;
		Globals.bottomOutputField.setForeground(OurColors.bottomOutputFieldForeHighlight);
	    }
	Globals.bottomOutputField.setText(outputString);
		
	if (stopProgram && Globals.programRunner != null || warningError && Globals.haltOnErrors)
	    {
		Globals.programRunner = null;
		Globals.isThreaded = false;
		Globals.mainGUI.run.setEnabled(true);
	    }
    }

    // Operand Code 1: RXY: Load the register R with the bit pattern found in the memory cell whose address is XY
    public static void LoadAddress()
    {
	temp = 16 * instruction3 + instruction4;

	if (Globals.MemoryAddress[2 * temp] == -1)
	    {
		Globals.MemoryAddress[2 * temp] = 0;
		noValue = true;
	    }
	if (Globals.MemoryAddress[2 * temp + 1] == -1)
	    {
		Globals.MemoryAddress[2 * temp + 1] = 0;
		noValue = true;
	    }
	Globals.RegisterAddress[2 * instruction2] = Globals.MemoryAddress[2 * temp];
	Globals.RegisterAddress[2 * instruction2 + 1] = Globals.MemoryAddress[2 * temp + 1];

	updateRegisterAddresses(instruction2);
	outputString = "Register " + outchar2 + " loaded with bit pattern found at memory address " + Globals.toHex(temp);
    }

    // Operand Code 2: RXY: Load the register R with the bit pattern XY
    public static void Load()
    {
	Globals.RegisterAddress[2 * instruction2] = instruction3;
	Globals.RegisterAddress[2 * instruction2 + 1] = instruction4;

	updateRegisterAddresses(instruction2);
	outputString = "Register " + outchar2 + " loaded with bit pattern " + outchar3 + outchar4;
    }

    // Operand Code 3: RXY: Store the bit pattern found in register R in the memory cell whose address is XY
    public static void Store() {

	// this is a base conversion from hex to decimal (hence the 16)
	temp = (16 * instruction3) + instruction4;

	System.out.println("instruction3="+instruction3+"instruction4="+instruction4);
	System.out.println("temp="+temp);
	System.out.println("Globals.RegisterAddress[2 * instruction2]="+Globals.RegisterAddress[2 * instruction2]);
	System.out.println("Globals.RegisterAddress[2 * instruction2+1]="+Globals.RegisterAddress[2 * instruction2+1]);

	if( temp % 2 == 1 ){
	    if (Globals.MemoryAddress[2 * temp - 1] == -1 
		|| Globals.MemoryAddress[2 * temp - 2] == -1){
		
		Globals.MemoryAddress[2 * temp - 1] = 0;
		Globals.MemoryAddress[2 * temp - 2] = 0;
	    }
	}else{
	    if( Globals.MemoryAddress[2 * temp + 2] == -1
		|| Globals.MemoryAddress[2 * temp + 3] == -1){
		Globals.MemoryAddress[2 * temp + 2] = 0;
		Globals.MemoryAddress[2 * temp + 3] = 0;
	    }
	}

	Globals.MemoryAddress[2 * temp] = Globals.RegisterAddress[2 * instruction2];
	Globals.MemoryAddress[2 * temp + 1] = Globals.RegisterAddress[2 * instruction2 + 1];

	Globals.updateTextfields(temp / 2);
	outputString = "Bit pattern found in in register " + outchar2
	    + " stored at memory address " + Globals.toHex(temp);
    }

    // Operand Code 4: 0RS: Move the bit pattern found in register R to register S
    public static void Move()
    {
	Globals.RegisterAddress[2 * instruction4] = Globals.RegisterAddress[2 * instruction3];
	Globals.RegisterAddress[2 * instruction4 + 1] = Globals.RegisterAddress[2 * instruction3 + 1];

	updateRegisterAddresses(instruction4);
	outputString = "Bit pattern found in in register " + outchar3
	    + " moved to register " + 4;
    }

    // Operand Code 5: RST: Add the bit pattern found in register S and T as though they were two's complement representations and leave the result in register R
    public static void Add()
    {
	int value1, value2, finalAns;
	value1 = (Globals.RegisterAddress[2 * instruction3] << 4)
	    + Globals.RegisterAddress[2 * instruction3 + 1];
	value2 = (Globals.RegisterAddress[2 * instruction4] << 4)
	    + Globals.RegisterAddress[2 * instruction4 + 1];

	// check for sign bit and convert to normal number if necessary
	if ((value1 >> 7) == 1)
	    {
		value1 = ((value1 ^ 127) & 127) + 1;
		// let's negate the value now
		value1 = -value1;
	    }

	// check for sign bit and convert to normal number if necessary
	if ((value2 >> 7) == 1)
	    {
		value2 = ((value2 ^ 127) & 127) + 1;
		// let's negate the value now
		value2 = -value2;
	    }

	// add the two values together
	finalAns = value1 + value2;


	// now let's convert back to two's complement if necessary
	if (finalAns < 0)
	    {
		finalAns = -finalAns;
		finalAns = ((finalAns ^ 127) & 127) + 1;
		// add sign bit back
		finalAns = finalAns + (1 << 7);
	    }

	outputString = "Bit patterns found in in registers " + outchar3 + " and "
	    + outchar4 + " added together as two's complement and stored in register " + outchar2;
		

	// check if overflow occurred
	if ((finalAns >> 8) != 0)
	    {
		finalAns %= 256;
		outputString = "WARNING: Overflow occurred!  " + outputString;
		warningError = true;
	    }
	Globals.RegisterAddress[2 * instruction2] = (byte)(finalAns >> 4);
	Globals.RegisterAddress[2 * instruction2 + 1] = (byte)(finalAns % 16);
					
	updateRegisterAddresses(instruction2);
    }

    // Operand Code 6: RST: Add the bit patterns in registers S and T as though they represented values in the floatin-point notation and leave the floating point result in register R
    public static void AddFloat()
    {
	int firstExp, secondExp, firstMan, secondMan;
	int answerExp = 0, answerSign = 0, answerMan;
	boolean overflow = false;

	firstExp = Globals.RegisterAddress[2 * instruction3];
	firstExp = (firstExp & 7); // chop of the sign bit, finds exponent + 4

	firstMan = Globals.RegisterAddress[2 * instruction3 + 1];	// mantissa
	firstMan = firstMan << firstExp;
	if (Globals.RegisterAddress[2 * instruction3] >> 3 == 1)	// if number is negative, make it negative
	    firstMan = -firstMan;

	secondExp = Globals.RegisterAddress[2 * instruction4];
	secondExp = (secondExp & 7); // chop of the sign bit, finds exponent + 4

	secondMan = Globals.RegisterAddress[2 * instruction4 + 1];	// mantissa
	secondMan = secondMan << secondExp;
	if (Globals.RegisterAddress[2 * instruction4] >> 3 == 1)
	    secondMan = -secondMan;

	// add the two values together
	answerMan = firstMan + secondMan;

	// the values have been added, now convert the result back to the floating point format

	// check if answer is negative
	if (answerMan < 0)
	    {
		answerSign = 1;	// answer is negative
		answerMan = -answerMan;
	    }

	// find the exponent and reduce the mantissa to an acceptable size.
	// while (answerMan > 15 || (answerExp < 7 && (answerMan >> 3) != 1) )
	while (answerMan > 15)
	    {
			
		answerExp++;	// increment exponent
		answerMan = answerMan >> 1;
	    }
		
	// make sure the exponent isn't too big, if it is reduce it and set overflow to true
	if (answerExp > 7)
	    {
		answerExp = 7;
		overflow = true;
	    }
				
	outputString = "Bit patterns found in in registers " + outchar3 + " and "
	    + outchar4 + " added together as floating point values and stored in register " + outchar2;
	if (overflow)
	    {
		outputString = "WARNING: Overflow occurred!  " + outputString;
		warningError = true;
	    }
		
	Globals.RegisterAddress[2 * instruction2] = (byte)(answerExp + (answerSign << 3));
	Globals.RegisterAddress[2 * instruction2 + 1] = (byte)answerMan;

	updateRegisterAddresses(instruction2);
    }

    // Operand Code 7: RST: Or the bit patterns in registers S and T and place the result in register R
    public static void Or()
    {
	temp = ((Globals.RegisterAddress[2 * instruction3] << 4)
		+ Globals.RegisterAddress[2 * instruction3 + 1])
	    | ((Globals.RegisterAddress[2 * instruction4] << 4)
	       + Globals.RegisterAddress[2 * instruction4 + 1]);

	Globals.RegisterAddress[2 * instruction2] = (byte)(temp >> 4);
	Globals.RegisterAddress[2 * instruction2 + 1] = (byte)(temp % 16);

	updateRegisterAddresses(instruction2);
	outputString = "Bit patterns found in in registers " + outchar3 + " and "
	    + outchar4 + " or-ed and stored in register " + outchar2;
    }

    // Operand Code 8: RST: And the bit patterns in register S and T and place the result in register R
    public static void And()
    {
	temp = ((Globals.RegisterAddress[2 * instruction3] << 4)
		+ Globals.RegisterAddress[2 * instruction3 + 1])
	    & ((Globals.RegisterAddress[2 * instruction4] << 4)
	       + Globals.RegisterAddress[2 * instruction4 + 1]);

	Globals.RegisterAddress[2 * instruction2] = (byte)(temp >> 4);
	Globals.RegisterAddress[2 * instruction2 + 1] = (byte)(temp % 16);

	updateRegisterAddresses(instruction2);
	outputString = "Bit patterns found in in registers " + outchar3 + " and "
	    + outchar4 + " and-ed and stored in register " + outchar2;
    }

    // Operand Code 9: RST: Exclusive Or the bit patterns in registers S and T and place the result in register R
    public static void ExclusiveOr()
    {
	temp = ((Globals.RegisterAddress[2 * instruction3] << 4)
		+ Globals.RegisterAddress[2 * instruction3 + 1])
	    ^ ((Globals.RegisterAddress[2 * instruction4] << 4)
	       + Globals.RegisterAddress[2 * instruction4 + 1]);

	Globals.RegisterAddress[2 * instruction2] = (byte)(temp >> 4);
	Globals.RegisterAddress[2 * instruction2 + 1] = (byte)(temp % 16);

	updateRegisterAddresses(instruction2);
	outputString = "Bit patterns found in in registers " + outchar3 + " and "
	    + outchar4 + " xor-ed and stored in register " + outchar2;
    }

    // Operand Code A: R0X: Rotate the bit pattern in register R one bit to the right X times.  Each time place the bit that started at the low-order end at the high order end
    public static void Rotate()
    {
	temp = (Globals.RegisterAddress[2 * instruction2] * 16
		+ Globals.RegisterAddress[2 * instruction2 + 1]);
	int i = instruction4;

	for (; i > 0; i--){
		
	    if ((temp % 2) == 1){
			
		temp = temp >> 1;
		temp += 128;
	    }
	    else
		temp = temp >> 1;
	}

	Globals.RegisterAddress[2 * instruction2] = (byte)(temp >> 4);
	Globals.RegisterAddress[2 * instruction2 + 1] = (byte)(temp % 16);

	updateRegisterAddresses(instruction2);
	outputString = "Bit pattern found in in register " + outchar2
	    + " rotated to the right " + outchar4 + " times";
    }

    // Operand Code B: RXY: Jump to the instruction located in the
    // memory cell at address XY if the bit pattern in register R
    // is equal to the bit pattern in register number
    // 0. Otherwise, continue with the normal sequence of
    // execution.
    public static void Jump(){
	
	if (Globals.RegisterAddress[2 * instruction2] == Globals.RegisterAddress[0] 
	    && Globals.RegisterAddress[2 * instruction2 + 1] == Globals.RegisterAddress[1]){
		 
	    Globals.ProgramCounter = 16 * instruction3 + instruction4;
	    updateProgramCounter();
		
	    outputString = "Bit pattern found in in register " + outchar2 + " equals value in register 0, "
		+ " program counter set to value " + outchar3 + outchar4;
			
	    if (Globals.ProgramCounter % 2 == 1){
		Globals.RegisterAddress[2 * instruction2] %= 16;
		outputString = "WARNING: Program counter set to odd number!  " + outputString;
		Globals.stopProgram = true;
	    }
		
	}else{
	    outputString = "Bit pattern found in register " + outchar2 + " does not equal value in register 0, "
		+ " program counter not changed";
	}
    }

    // Operand Code C: Halt execution
    public static void Halt()
    {
	outputString = "Program Halted";
	Globals.stopProgram = true;
    }

    // Operand Code D: RXY: Jump to the instruction located in the
    // memory cell at address XY if the bit pattern in register R
    // is greater than the bit pattern in register number
    // 0. Otherwise, continue with the normal sequence of
    // execution.
    public static void JumpGreater(){
	int compareValue= Globals.RegisterAddress[2 * instruction2] << 4;
	compareValue+= Globals.RegisterAddress[2 * instruction2 + 1];
	
	int zeroAddress=Globals.RegisterAddress[0] << 4;
	zeroAddress+=Globals.RegisterAddress[1];

	boolean jump=false;
	
	if( (compareValue & 128) < (zeroAddress & 128) ){
	    jump = true;
	}else if( (compareValue & 128) == (zeroAddress & 128) ){
	    if( (compareValue & 128) ==0 
		&& compareValue > zeroAddress){
		jump=true;
	    }
	    if( (compareValue & 128) !=0){
		
		compareValue = (~compareValue)+1;
		zeroAddress = (~zeroAddress)+1;
		
		if( zeroAddress > compareValue ){
		    jump=true;
		}
		
	    }
	}
	
	if(jump){
	    Globals.ProgramCounter = 16 * instruction3 + instruction4;
	    updateProgramCounter();
		
	    outputString = "Bit pattern found in register " + outchar2 
		+ " is greater than the value in register 0, "
		+ " program counter set to value " + outchar3 + outchar4;
			
	    if (Globals.ProgramCounter % 2 == 1){
		Globals.RegisterAddress[2 * instruction2] %= 16;
		outputString = "WARNING: Program counter set to odd number!  " + outputString;
		Globals.stopProgram = true;
	    }
		
	}else{
	    outputString = "Bit pattern found in in register " + outchar2 
		+ " is not greater than value in register 0, "
		+ " program counter not changed";
	}
    }

    public static void noOpCode()
    {
	outputString = "No operator code found, Program Halted";
	Globals.stopProgram = true;
    }
}
