/**
*
*		File Save Dialog
* Last edited: 1/30/2000, 1:29 AM by Rogan
*
* Revision history:
* 1/20/00 - added MailIO instantiation/declaration
*           added call to send() method to the send button
*           pressing Send now sends, then closes the dialog
*
* 1/21/00 - moved MailIO.send() call to SaveDialog.send() method
*			added method String findCode() to get code from input
*			fields.  SaveDialog.send() ready to pass 3 String args to
*			MainIO.send().
*			added errorDialog for invalid email addresses.
* 1/30/00 - 'x' in corner of window closes it now.
*
*
**/


import java.awt.*;
import java.awt.event.*;
import java.io.IOException;


public class SaveDialog extends Dialog
{
	private boolean commentsVisible = false;

	private ActionGrabber actionHandler = new ActionGrabber();

	private Panel gridBagPanel = new Panel();
	private Panel buttonPanel = new Panel();
	private TextArea commentsArea = new TextArea();

	private TextField emailField   = new TextField();
	private TextField subjectField = new TextField();

	private Button comments = new Button("Comments");
	private Button cancel   = new Button("Cancel");
	private Button send     = new Button("Send");

	private Label email = new Label("Email:");
	private Label subject = new Label("Subject:");

	private Frame owner;
	//private MailIO mailer;

	public SaveDialog(Frame towner)
	{
		super(towner);
		this.owner = towner;
		addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e){e.getWindow().dispose();}});
		//mailer = new MailIO(new String("brock@engr.orst.edu"));
		setTitle("Save");
		setModal(true);
		setResizable(true);
		initWidgits();
		addActionStuff();
	}// end SaveDialog()

	private void initWidgits()
	{
		setLayout(new BorderLayout());
		setColor();

		GridBagLayout gridBag = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		gridBagPanel.setLayout(gridBag);

		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.weightx = 0;
		constraints.weighty = 0;

		gridBagPanel.add(email,constraints);

		constraints.gridx = 0;
		constraints.gridy = 1;

		gridBagPanel.add(subject,constraints);

		constraints.gridwidth = 2;
		constraints.gridx = 1;
		constraints.gridy = 0;
		constraints.weightx = 100;
		constraints.fill = GridBagConstraints.HORIZONTAL;


		gridBagPanel.add(emailField,constraints);

		constraints.gridwidth = 2;
		constraints.gridx = 1;
		constraints.gridy = 1;

		gridBagPanel.add(subjectField,constraints);

		constraints.gridx = 0;
		constraints.gridy = 2;
		constraints.gridwidth = 3;
		constraints.weightx = 0;
		constraints.fill = GridBagConstraints.NONE;

		buttonPanel.add(comments);
		buttonPanel.add(send);
		buttonPanel.add(cancel);

		gridBagPanel.add(buttonPanel,constraints);


		commentsArea.setVisible(false);

		add(gridBagPanel,"North");
		add(commentsArea,"Center");

	}// end initWidgits()


	private void setColor()
	{
		this.setBackground(OurColors.outer);
		this.setForeground(OurColors.titles);

		gridBagPanel.setBackground(OurColors.outer);
		gridBagPanel.setForeground(OurColors.titles);
		buttonPanel.setBackground(OurColors.outer);
		buttonPanel.setForeground(OurColors.titles);

		email.setBackground(OurColors.outer);
		email.setForeground(OurColors.titles);
		subject.setBackground(OurColors.outer);
		subject.setForeground(OurColors.titles);

		commentsArea.setBackground(OurColors.fieldBack);
		commentsArea.setForeground(OurColors.fieldFore);
		emailField.setBackground(OurColors.fieldBack);
		emailField.setForeground(OurColors.fieldFore);
		subjectField.setBackground(OurColors.fieldBack);
		subjectField.setForeground(OurColors.fieldFore);

		comments.setBackground(OurColors.buttonBack);
		comments.setForeground(OurColors.buttonFore);
		cancel.setBackground(OurColors.buttonBack);
		cancel.setForeground(OurColors.buttonFore);
		send.setBackground(OurColors.buttonBack);
		send.setForeground(OurColors.buttonFore);
	}// end setColor()

	public void showComments(boolean b)
	{
		commentsVisible = b;

		if(b)
		{
			this.setSize(getBounds().width,getBounds().height+100);
			commentsArea.setVisible(b);
			validate();
		}else{
			commentsArea.setVisible(b);
			setSize(getBounds().width,getBounds().height-100);
			validate();
		}


	}// end showComments()

	private void send()
	{
		//mailer = new MailIO(new String("brock@engr.orst.edu"));

		Globals.lastEmail = emailField.getText();
		String emailAddress = emailField.getText();
		String subject = subjectField.getText();
		String commentsStr;
		String message = new String("");

		if(commentsVisible)
		{
			commentsStr = commentsArea.getText();
			message = message + commentsStr +"\n\n";
		}

		message = message + findCode();
/*
		try {
			// call mailer.send() with String emailAddress (for target email)
			// String subject, and String message, this not yet implemented.
			mailer.send(emailAddress, subject, message);

		} catch (IOException ioe) {
			System.out.println(ioe);
		}
*/
	}// end save()

	public String getComments()
	{
		return commentsArea.getText();
	}// end getComments()

	public void setEmail(String email)
	{
		emailField.setText(email);
	}// end setEmail()

	private String findCode()
	{
		Commands.getInput();
		String code = new String("--Code--\n");
		for(int x=0; x < Globals.MEMSIZE / 2;x++)
		{
			code = code + Globals.toHex(Globals.MemoryAddress[4*x])
				   + Globals.toHex(Globals.MemoryAddress[4*x+1])
				   + Globals.toHex(Globals.MemoryAddress[4*x+2])
				   + Globals.toHex(Globals.MemoryAddress[4*x+3]) +"\n";
		}

		return code + "--EndCode--";
	}// end findCode()


	private void closeWin()
	{
		this.dispose();
	}// end closeWin()

	public void promptSave()
	{
		System.out.println("Showing dialog..");
		this.show();
	}

	public void addActionStuff()
	{
		cancel.addActionListener( actionHandler );
		comments.addActionListener( actionHandler );
		send.addActionListener( actionHandler );
	}// end addActionStuff()

	class ActionGrabber implements ActionListener
	{

		public void actionPerformed(ActionEvent e)
		{
			String command = e.getActionCommand();

			if( command.equals("Cancel"))
			{
				System.out.println("SaveDialog -> Cancel pressed");
				closeWin();
			}
			else if( command.equals("Comments"))
			{
				showComments(!commentsVisible);
				System.out.println("SaveDialog -> Comments pressed");
			}
			else if( command.equals("Send"))
			{
				System.out.println("SaveDialog -> Send pressed");
				String tmpStr = emailField.getText();
				if(tmpStr.indexOf('@') != -1)
				{
					send();
					closeWin();
				}
				else
					Globals.errorDialog("Email address is not valid");


			}

		}// end actionPerformed()
	}// end inner class actionGrabber
}// end class Dialog
